import matplotlib
matplotlib.use("AGG")

import momentsml.tools
import momentsml.learn
import momentsml

import config
import numpy as np
import os

import plot_3_valw

import logging
logging.basicConfig(format=config.loggerformat, level=logging.DEBUG)
logger = logging.getLogger(__name__)



for subfield in config.great3.subfields:

	logger.info("Working on subfield {}".format(subfield))

	valname = "pred_{}".format(config.datasets["valid-overall"]) # Too messy to add everything here.
	predcatpath = config.great3.subpath(subfield, "val", valname + ".pkl")
	figpredcatpath = config.great3.subpath(subfield, "val", valname + ".png")

	skip = False # Set to true to just make the plot without recomputing predictions
	if skip == False:
		catpath = config.great3.subpath(subfield, "simmeas", config.datasets["valid-overall"], "groupmeascat.pkl")
		cat = momentsml.tools.io.readpickle(catpath)
		#print momentsml.tools.table.info(cat)
	
	
		sheartraindir = config.great3.subpath(subfield, "ml", config.datasets["train-shear"])
		predcat = momentsml.learn.tenbilacrun.predict(cat, config.shearconflist , sheartraindir)
	
		if len(config.weightconflist) > 0:
		
			weighttraindir = config.great3.subpath(subfield, "ml", config.datasets["train-weight"])
			predcat = momentsml.learn.tenbilacrun.predict(predcat, config.weightconflist , weighttraindir)
		
		momentsml.tools.io.writepickle(predcat, predcatpath)
	else:
		predcat = momentsml.tools.io.readpickle(predcatpath)

	shearconfnames = momentsml.learn.tenbilacrun.confnames(config.shearconflist)
	for (confname, conf) in zip(shearconfnames, config.shearconflist):

		predcatcopy = predcat.copy() # As plots modifies it.
		if "s2" in conf[0] or "g2" in conf[0]:
			component = 2
		else:
			component = 1
		
		figpredcatpath = config.great3.subpath(subfield, "val", valname + "_" + str(component) + ".png")
		plot_3_valw.plot(predcatcopy, component, filepath=figpredcatpath)
		



import config
import measfcts
import os
import numpy as np
import matplotlib.ticker as ticker

import momentsml.plot
from momentsml.tools.feature import Feature
import matplotlib.pyplot as plt

import logging
logger = logging.getLogger(__name__)



from matplotlib import rc
rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
## for Palatino and other serif fonts use:
#rc('font',**{'family':'serif','serif':['Palatino']})
rc('text', usetex=True)

###############################

weights = True # Run on a validation catalog with weights, or without weights ?

###############################

select = True # Select accordign to <SNR> (only if without weights, is set to False otherwise)
regressmethod = 2 # Only matter when runnign with weights. 2 should be used, as we care about each PSF equally.

###############################


if weights is True:
	select = False
	valname = config.wvalname
else:
	regressmethod = 1
	valname = config.valname

valcat = os.path.join(config.valdir, valname + ".pkl")
cat = momentsml.tools.io.readpickle(valcat)


widescale=False
showlegend=False
if "sum55" in config.valname:
	text = "Ignoring the variability of the PSF"
	showlegend=True
	widescale=True
if "sum77" in config.valname or "pos" in config.valname:
	text = "Using field coordinates as features"
if "sum88" in config.valname or "mom" in config.valname:
	text = "Using PSF moments as features"


if widescale:
	lim = 1e0
else:
	lim = 1e-1


if select:
	momentsml.tools.table.addstats(cat, "snr")
	s = momentsml.tools.table.Selector("snr_mean > 10", [
		("min", "snr_mean", 10.0),
	])
	cat = s.select(cat)
#exit()

"""
for comp in ["1","2"]:

	# If no weights are in the catalog (or not yet), we add ones
	if not "pre_s{}w".format(comp) in cat.colnames:
		
		# First putting all weights to 1.0:
		cat["pre_s{}w".format(comp)] = np.ones(cat["adamom_g1"].shape)
		
	cat["pre_s{}w_norm".format(comp)] = cat["pre_s{}w".format(comp)] / np.max(cat["pre_s{}w".format(comp)])

	momentsml.tools.table.addrmsd(cat, "pre_s{}".format(comp), "tru_s{}".format(comp))
	momentsml.tools.table.addstats(cat, "pre_s{}".format(comp), "pre_s{}w".format(comp))
	cat["pre_s{}_wbias".format(comp)] = cat["pre_s{}_wmean".format(comp)] - cat["tru_s{}".format(comp)]
"""



fwhmpersigma = 2.3548
cat["tru_psf_fwhm"] = cat["tru_psf_sigma"] * fwhmpersigma

tru_psf_g1 = Feature("tru_psf_g1", -0.27, 0.27, nicename=r"PSF $\varepsilon_1$")
tru_psf_g2 = Feature("tru_psf_g2", -0.02, 0.27, nicename=r"PSF $\varepsilon_2$")
tru_psf_fwhm = Feature("tru_psf_fwhm", 4.1, 5.35, nicename=r"PSF FWHM [pix]")


def make_plot(ax, featbin, showlegend=False):
	ax.axhline(0.0, color='gray', lw=0.5)	
	if weights is True:
		momentsml.plot.mcbin.mcbin(ax, cat, Feature("tru_s1"), Feature("pre_s1", rea="all"), featbin, featprew=Feature("pre_s1w", rea="all"), comp=1, regressmethod=regressmethod)
		momentsml.plot.mcbin.mcbin(ax, cat, Feature("tru_s2"), Feature("pre_s2", rea="all"), featbin, featprew=Feature("pre_s2w", rea="all"), comp=2, showbins=False, showlegend=showlegend, regressmethod=regressmethod)
	else:
		momentsml.plot.mcbin.mcbin(ax, cat, Feature("tru_s1"), Feature("pre_s1", rea="all"), featbin, comp=1, regressmethod=regressmethod)
		momentsml.plot.mcbin.mcbin(ax, cat, Feature("tru_s2"), Feature("pre_s2", rea="all"), featbin, comp=2, showbins=False, showlegend=showlegend, regressmethod=regressmethod)	
	momentsml.plot.mcbin.make_symlog(ax, featbin, lim=lim)
	ax.set_xlabel(featbin.nicename)


fig = plt.figure(figsize=(10, 3.0))
plt.subplots_adjust(
	left  = 0.08,  # the left side of the subplots of the figure
	right = 0.99,    # the right side of the subplots of the figure
	bottom = 0.15,   # the bottom of the subplots of the figure
	top = 0.86,      # the top of the subplots of the figure
	wspace = 0.08,   # the amount of width reserved for blank space between subplots,
	                # expressed as a fraction of the average axis width
	hspace = 0.2,   # the amount of height reserved for white space between subplots,
					# expressed as a fraction of the average axis heightbottom=0.1, right=0.8, top=0.9)
	)

ax = plt.subplot(1, 3, 1)
make_plot(ax, tru_psf_g1)
ax.set_ylabel("Metric value")

ax = plt.subplot(1, 3, 2)
make_plot(ax, tru_psf_g2)
ax.set_yticklabels([])

ax = plt.subplot(1, 3, 3)
make_plot(ax, tru_psf_fwhm, showlegend=showlegend)
ax.set_yticklabels([])



fig.text(.005, .94, text, fontdict={"fontsize":12})

momentsml.plot.figures.savefig(os.path.join(config.valdir, valname + "_psf_biases"), fig, fancy=True)

plt.show()




"""
General helper functions for shape measurements
"""
import os
import glob
import re
import numpy as np
import galsim
import astropy

import logging
logger = logging.getLogger(__name__)



def simmeasdict(measdir, simparams):
	"""
	Function to help you explore available measurements of simulations obtained by :func:`momentsml.meas.run.onsims`
	for a given measdir and simparams.
	So this function is here to "glob" the random file names for you.
	
	:param measdir: See :func:`momentsml.meas.run.onsims`
	:param simparams: idem
	
	Regular expressions are used for this to avoid making optimistic assumptions about these filenames.
	A dict is returned whose keys are the simulated catalog names, and the corresponding entries
	are lists of filenames of the pkls with the measurements on the different realizations for each catalog. 
	
	An example of a dict that is returned::
		
		{'20141020T141239_E8gh8u':
			['20141020T141239_E8gh8u_0_galimg_meascat.pkl',
			 '20141020T141239_E8gh8u_1_galimg_meascat.pkl',
			 '20141020T141239_E8gh8u_2_galimg_meascat.pkl'],
		 '20141020T141239_9UhtkX':
		 	['20141020T141239_9UhtkX_0_galimg_meascat.pkl',
			 '20141020T141239_9UhtkX_1_galimg_meascat.pkl',
			 '20141020T141239_9UhtkX_2_galimg_meascat.pkl']
		}

	"""
	
	incatfilepaths = sorted(glob.glob(os.path.join(measdir, simparams.name, "*_galimg_meascat.pkl")))
	basenames = map(os.path.basename, incatfilepaths)
	
	if len(incatfilepaths) == 0:
		raise RuntimeError("No meascat found in %s" % (os.path.join(measdir, simparams.name)))
	
	# Here is how they look: 20141020T141239_9UhtkX_0_galimg_meascat.pkl
	# We get a unique list of the catalog names, using regular expressions
	
	p = re.compile("(\w*_\w*)_(\d*)_galimg_meascat.pkl")
	
	matches = [p.match(basename) for basename in basenames]
	if None in matches:
		raise RuntimeError("Some files in '%s' have unexpected filenames, something is wrong!" % (measdir))
		
		
	namereatuples = [(match.group(0), match.group(1), match.group(2)) for match in matches]
	# This gives (full file name, catname, realization number)
	
	catnames = sorted(list(set([namereatuple[1] for namereatuple in namereatuples])))
	
	out = {}
	for catname in catnames:
		out[catname] = [namereatuple[0] for namereatuple in namereatuples if namereatuple[1] == catname]
	
	logger.info("Found %i catalogs, and %i realizations (%.1f per catalog, on average)" %
		(len(catnames), len(namereatuples), float(len(namereatuples))/float(len(catnames))))

	return out



def add_new_cols(cat, prefix="", floatcols=None, boolcols=None):
	"""Adds new columns to a catalogue in place, and sets the values to be masked.
	"""
	
	if floatcols != None:
		for col in floatcols:
			cat.add_column(astropy.table.MaskedColumn(name=prefix+col, dtype=float, length=len(cat)))
			cat[prefix+col].mask = [True] * len(cat)
	if boolcols != None:
		for col in boolcols:
			cat.add_column(astropy.table.MaskedColumn(name=prefix+col, dtype=bool, length=len(cat)))
			cat[prefix+col].mask = [True] * len(cat)
	


def mad(nparray):
	"""
	The Median Absolute Deviation
	http://en.wikipedia.org/wiki/Median_absolute_deviation

	Multiply this by 1.4826 to convert into an estimate of the Gaussian std.
	"""

	return np.median(np.fabs(nparray - np.median(nparray)))



def skystats(stamp):
	"""
	I measure some statistics of the pixels along the edge of an image or stamp.
	Useful to measure the sky noise, but also to check for problems. Use "mad"
	directly as a robust estimate the sky std.

	:param stamp: a galsim image, usually a stamp

	:returns: a dict containing "std", "mad", "mean" and "med"
	
	Note that "mad" is already rescaled by 1.4826 to be comparable with std.
	"""
	
	if isinstance(stamp, galsim.Image):
		a = stamp.array
		# Normally there should be a .transpose() here, to get the orientation right.
		# But in the present case it doesn't change anything, and we can skip it.
	else:
		a = stamp # Then we assume that it's simply a numpy array.
	
	edgepixels = np.concatenate([
			a[0,1:], # left
			a[-1,1:], # right
			a[:,0], # bottom
			a[1:-1,-1] # top
			])
	assert len(edgepixels) == 2*(a.shape[0]-1) + 2*(a.shape[0]-1)

	# And we convert the mad into an estimate of the Gaussian std:
	return {
		"std":np.std(edgepixels), "mad": 1.4826 * mad(edgepixels),
		"mean":np.mean(edgepixels), "med":np.median(edgepixels),
		"stampsum":np.sum(a)
		}



#!/usr/bin/env python

from distutils.core import setup

setup(
	name='tenbilac',
	version='1.0dev',
	packages=['tenbilac'],
	description='Inverse regression network',
	long_description=open('README.md').read(),
	author='Malte Tewes',
	license='GPLv3',
	url='http://github.com/mtewes/tenbilac'
)




"""
This is Tenbilac!
Net represent a "simple" network. See WNet if you're looking for weight(ed) predictions.
"""

import numpy as np

import logging
logger = logging.getLogger(__name__)

from . import layer
from . import utils
from . import act
from . import data

class Net():
	"""
	Object representing a network made out of one or several hidden layers.
	"""
	
	def __init__(self, ni, nhs, no=1, onlyid=False, actfctname="tanh", oactfctname="iden", multactfctname="iden", name=None, inames=None, onames=None):
		"""
		:param ni: Number of input features
		:param nhs: Numbers of neurons in hidden layers. Times -1 if you want product units in that layer.
		:type nhs: tuple
		:param no: Number of ouput neurons
		:param onlyid: Set this to true if you want identity activation functions on all layers
			(useful for debugging). Note that this overrides both actfctname and oactfctname!
			
		:param actfctname: name of activation function for hidden layers
		:param oactfctname: idem for output layer
			
		:param name: if None, will be set automatically
		:type name: string
		
		:param inames: a list of names (strings) for the input nodes, to be used, e.g., in checkplots.
			These names have a purely decorative function, and are optional.
		:param onames: idem, for the ouptut nodes.
		
		
		"""
	
		self.ni = ni
		self.nhs = nhs
		self.no = no
		self.name = name
		
		# We take care of the inames and onames:
		if inames is None:
			self.inames = ["i_"+str(i) for i in range(self.ni)]
		else:
			self.inames = inames
			if len(self.inames) != self.ni:
				raise RuntimeError("Your number of inames is wrong")
		if onames is None:
			self.onames = ["o_"+str(i) for i in range(self.no)]
		else:
			self.onames = onames
			if len(self.onames) != self.no:
				raise RuntimeError("Your number of onames is wrong")
		
		iniarch = np.array([self.ni]+self.nhs+[self.no]) # Note that we do not save this. Layers might evolve dynamically in future!
		
		actfct = eval("act.{0}".format(actfctname)) # We turn the string actfct option into an actual function
		oactfct = eval("act.{0}".format(oactfctname)) # idem
		multactfct = eval("act.{0}".format(multactfctname)) # idem
		
		self.layers = [] # We build a list containing only the hidden layers and the output layer
		for (i, nh) in enumerate(self.nhs):
			if nh > 0:
				self.layers.append(layer.Layer(ni=iniarch[i], nn=nh, actfct=actfct, name="h"+str(i), mode="sum"))
			elif nh < 0: # a "mult"-layer
				self.layers.append(layer.Layer(ni=iniarch[i], nn=-nh, actfct=multactfct, name="h"+str(i), mode="mult"))
			else:
				raise ValueError("Cannot have 0 hidden nodes")
		# Adding the output layer:
		self.layers.append(layer.Layer(ni=self.nhs[-1], nn=no, actfct=oactfct, name="o"))
		
		if onlyid: # Then all layers get the Id activation function:
			for l in self.layers:
				l.actfct = act.iden
				
		logger.info("Built " + str(self))


	
	def __str__(self):
		"""
		A short string describing the network
		"""
		archtxt = str(self.ni) + "|" + "|".join(["{n}/{modecode}{actfct}".format(n=l.nn, modecode=l.modecode(), actfct=l.actfct.__name__) for l in self.layers])
		autotxt = "[{archtxt}={nparams}]".format(archtxt=archtxt, nparams=self.nparams())
		
		if self.name is None:
			return autotxt
		else:
			return "'{name}' {autotxt}".format(name=self.name, autotxt=autotxt)


	
	def report(self):
		"""
		Returns a text about the network parameters, useful for debugging.
		"""
		txt = ["="*120, str(self)]
		for l in self.layers:
			txt.append(l.report())
		txt.append("="*120)
		return "\n".join(txt)

	
	def resetcache(self):
		"""
		Purges the cache of every layer (e.g., to save memory before writing to file)
		"""
		for l in self.layers:
			l.resetcache()
	
	
	def save(self, filepath, resetcache=True):
		"""
		Saves self into a pkl file
		"""
		if resetcache:
			self.resetcache()
		utils.writepickle(self, filepath)		
	
	
	def nparams(self):
		"""
		Returns the number of parameters of the network
		"""
		return sum([l.nparams() for l in self.layers])
		
	
	def get_params_ref(self):
		"""
		Get a single 1D numpy array containing references to all network weights and biases.
		Note that each time you call this, you loose the "connection" to the ref from any previous calls.
		
		Warning: if you modify this code, pay attention to update the paramslice-related stuff as well!
		"""
		
		ref = np.empty(self.nparams())
		ind = 0
		
#		#First layer first, weights and biases.
#			for l in self.layers:
#				ref[ind:ind+(l.nn*l.ni)] = l.weights.flatten() # makes a copy
#				ref[ind+(l.nn*l.ni):ind+l.nparams()] = l.biases.flatten() # makes a copy
#				l.weights = ref[ind:ind+(l.nn*l.ni)].reshape(l.nn, l.ni) # a view
#				l.biases = ref[ind+(l.nn*l.ni):ind+l.nparams()] # a view
#				ind += l.nparams()
		
		# Starting at the end, biases before weights
		
		for l in self.layers[::-1]:
			
			ref[ind:ind+l.nn] = l.biases.flatten() # makes a copy
			ref[ind+l.nn:ind+l.nparams()] = l.weights.flatten() # makes a copy
			l.biases = ref[ind:ind+l.nn] # a view
			l.weights = ref[ind+l.nn:ind+l.nparams()].reshape(l.nn, l.ni) # a view
			ind += l.nparams()
			
			
		# Note that such tricks do not work, as indexing by indices creates copies:
		#indices = np.arange(self.nparams())
		#np.random.shuffle(indices)
		#return ref[indices]

		assert ind == self.nparams()
		return ref



	def get_paramlabels(self):
		"""
		Returns a list with labels describing the params. This is for humans and plots.
		Note that plots might expect these labels to have particular formats.
		"""
			
		paramlabels=[]
		ind = 0
		
		#if schema == 2:
		for l in self.layers[::-1]:
				
			paramlabels.extend(l.nn*["layer-{l.mode}-{l.name}_bias".format(l=l)])
			paramlabels.extend(l.nn*l.ni*["layer-{l.mode}-{l.name}_weight".format(l=l)])
		
		assert len(paramlabels) == self.nparams()
		
		return paramlabels
		


	def addnoise(self, **kwargs):
		"""
		Adds random noise to all parameters.
		"""
		
		logger.info("Adding noise to network parameters ({})...".format(str(kwargs)))
		
		for l in self.layers:
			l.addnoise(**kwargs)
			
			
	def setidentity(self, **kwargs):
		"""
		Adjusts the network parameters so to approximatively get an identity relation
		between the ith output and the ith input (for each i in the outputs).
		
		This should be a good starting position for "calibration" tasks. Example: first
		input feature is observed galaxy ellipticity e1, and first output is shear g1.
		"""

		for l in self.layers:
			if l.nn < self.no or self.ni < self.no:
				raise RuntimeError("Network is too small for setting identity!")
			
		for l in self.layers:
				l.setidentity(**kwargs)
			
		logger.info("Done with setting weights to identity.")
			

	def run(self, inputs):
		"""
		Propagates input through the network "as fast as possible".
		This works for 1D, 2D, and 3D inputs, see layer.run().
		Note that this forward-running does not care about the fact that some of the inputs might be masked!
		In fact it **ignores** the mask and will simply compute unmasked outputs.
		Use predict() if you have masked inputs and want to "propagate" the mask appropriately.
		"""
		
		# Normally we should go ahead and see if it fails, but in this particular case it's more helpful to test ahead:
		
		if inputs.ndim == 3:
			if inputs.shape[1] != self.ni:
				raise ValueError("Inputs with {ni} features (shape = {shape}) are not compatible with {me}".format(ni=inputs.shape[1], shape=inputs.shape, me=str(self)))
		elif inputs.ndim == 2:
			if inputs.shape[0] != self.ni:
				raise ValueError("Inputs with {ni} features (shape = {shape}) are not compatible with {me}".format(ni=inputs.shape[0],shape=inputs.shape, me=str(self)))

		
		output = inputs
		for l in self.layers:
			output = l.run(output)
		return output
	
			
	
	def predict(self, inputs):
		"""
		We compute the outputs from the inputs using self.run, but here we do take care of the potential mask.
		
		This is never used during the training phase.
		
		:param inputs: a (potentially masked) 3D array
		
		:returns: a 3D array, appropriately masked
		
		"""
		
		logger.info("Predicting with input = {intype} of shape {inshape}".format(
			intype=str(type(inputs)), inshape=str(inputs.shape)))

		if inputs.ndim != 3:
			raise ValueError("Sorry, I only accept 3D input")

		(inputs, outputsmask) = data.demask(inputs, no=self.no)
		
		# We can simply run the network with the unmasked inputs:
		
		logger.info("Running the actual predictions...")
		outputs = self.run(inputs)
		
		# And now mask these outputs, if required:
		
		if outputsmask is not None:
			outputs = np.ma.array(outputs, mask=outputsmask)
		
		return outputs
		





import numpy as np
import tenbilac

import logging
logging.basicConfig(level=logging.INFO)

import matplotlib.pyplot as plt

"""
# Test with 5 reas, 2 nodes, 3 cases.

a = np.random.randn(5*2*3).reshape(5, 2, 3)
mask = np.zeros(5*2*3).reshape(5, 2, 3)
a = np.ma.array(a, mask=mask)

# First node and case has one crazy value:
a[0,0,0] = 1.0e8

print "mean"
print np.mean(a, axis=0)

a.mask[0,0,0] = True

print np.mean(a, axis=0)

# What happens if all realizations are masked :

a.mask[:,0,0] = True

print np.mean(a, axis=0)

print "std"
print np.std(a, axis=0)
a.mask[:,0,0] = False
print np.std(a, axis=0)

# And if only one rea is available:

a.mask[1:,0,0] = True
print np.std(a, axis=0)

# OK it gives 0.0. All as exected.
"""

# Testing tile: Good, works as expected.
a = np.ma.array([1, 2, 3, 4], mask=[True, False, False, False])

print a
print np.tile(a, 3)





"""
# HUGE WARNING: NP.RAVEL() SILENTLY IGNORES THE MASK !!!
a = np.ma.array([[1, 2], [3, 4]], mask=[[True, False], [False, False]])

print a

print np.ravel(a)

print a.flatten()

"""


"""
# Test of dot on masked arrays:

a = np.ma.array([[1, 2], [3, 4]], mask=[[True, False], [False, False]])
b = np.ma.array([[1, 2], [3, 4]], mask=[[False, False], [False, False]])

print "a = "
print a

print "b = "
print b

print "dot = "
print np.ma.dot(a, b, strict=True) # This works, but only for 2D arrays !
# See code of np.ma.dot, it's quite explicit.

"""


"""
ni = 3
ngal = 4
nrea = 1
no = 1

net = tenbilac.net.Tenbilac(ni=ni, nhs=[3, 3], onlyid=True)

net.setidentity()
net.addnoise()
print net.report()

inputs = np.random.randn(nrea* ni * ngal).reshape((nrea, ni, ngal))
r_mask = 1.0
inputs = np.ma.masked_outside(inputs, -r_mask, r_mask)

print "Input:"
print inputs.shape
print inputs


#output = net.run(inputs)

output = net.layers[0].run(inputs)
#output = net.layers[1].run(output)
#output = net.layers[2].run(output)


print "Output:"
print output.shape
print output

"""






import momentsml
import momentsml.learn
import os
import numpy as np

import logging


# A simple logger format:
loggerformat='PID %(process)06d | %(asctime)s | %(levelname)s: %(name)s(%(funcName)s): %(message)s'
# Try this one, it's colorful:
#loggerformat='\033[1;31m%(levelname)s\033[1;0m: %(name)s(%(funcName)s): \033[1;21m%(message)s\033[1;0m'

logging.basicConfig(format=loggerformat,level=logging.INFO)

# Give a path to an existing directory in which everything can be written (will get large!)

workdir = "/vol/fohlen11/fohlen11_1/mtewes/MomentsML/fiducial"

simdir = os.path.join(workdir, "sim")
simmeasdir = os.path.join(workdir, "simmeas")
traindir = os.path.join(workdir, "train")
valdir = os.path.join(workdir, "val")


stampsize = 64 # used for measuring
drawstampsize = stampsize # Used for drawing


datasets = {

	# Fot the fixed-PSF experiment (Section 6)
	"si":"si-1",
	"tp":"tp-1",
	"vp":"vp-1",
	"tw":"tw-1",
	"vo":"vo-1",


	# For the variable-PSF experiments (Section 7)
	#"si":"si-varpsf-1",
	#"tp":"tp-varpsf-1", # To speed up, use this one only after a pretraining on "tp-varpsf-1-pretrain", as described in the paper.
	#"tp":"tp-varpsf-1-pretrain", # Special dataset for the pre-training of the point-estimator, without shear, and structured for ellipticity
	#"vp":"vp-varpsf-1",
	#"tw":"tw-varpsf-1",
	#"vo":"vo-varpsf-1",	
	#"si":"si-varpsf-1",
}

# Configuration of the point-estimates

shearconflist = [
	# Typically uncomment a single line at a time to train a component,
	# and then uncomment the 2 lines of both components for the predictions.
	
	# For constant PSF experiements:
	("mlconfig/ada5s1f.cfg", "mlconfig/sum55.cfg"), # <--- We use those
	("mlconfig/ada5s2f.cfg", "mlconfig/sum55.cfg")
	
	# For the variable-PSF experiments:
	
	#    Using the field position as features:
	#("mlconfig/ada7s1f-varpsf-pos.cfg", "mlconfig/sum1010pos.cfg"),
	#("mlconfig/ada7s2f-varpsf-pos.cfg", "mlconfig/sum1010pos.cfg")
	
	#    Using size and ellipticity of the PSF as features:
	#("mlconfig/ada8s1f-varpsf-mom.cfg", "mlconfig/sum1010mom.cfg"),
	#("mlconfig/ada8s2f-varpsf-mom.cfg", "mlconfig/sum1010mom.cfg")
	
]

# Similar configuration for the weight estimator:

weightconflist = [
	("mlconfig/ada5s1wf.cfg", "mlconfig/sum5w.cfg"),
	("mlconfig/ada5s2wf.cfg", "mlconfig/sum5w.cfg")
	
	#("mlconfig/ada7s1wf-varpsf-pos.cfg", "mlconfig/sum10w.cfg"),
	#("mlconfig/ada7s2wf-varpsf-pos.cfg", "mlconfig/sum10w.cfg")
	
	#("mlconfig/ada8s1wf-varpsf-mom.cfg", "mlconfig/sum10w.cfg"),
	#("mlconfig/ada8s2wf-varpsf-mom.cfg", "mlconfig/sum10w.cfg")

]


#######################################################################################################################################

# Some automatically generated names, used for saving figures etc depending on your above settings.
# Not meant to be changed.

sconfname = os.path.splitext(os.path.basename(shearconflist[0][1]))[0] # extracts e.g. "sum55"
valname = "{}_with_{}_on_{}".format(datasets["tp"], sconfname, datasets["vp"])
wconfname = os.path.splitext(os.path.basename(weightconflist[0][1]))[0] # extracts e.g. "sum55w"
wvalname = "{}_and_{}_with_{}_{}_on_{}".format(datasets["tp"], datasets["tw"], sconfname, wconfname, datasets["vo"])

for d in [simdir, traindir, valdir]:
	if not os.path.exists(d):
		os.makedirs(d)



"""
http://stackoverflow.com/questions/3288595/multiprocessing-how-to-use-pool-map-on-a-function-defined-in-a-class
"""

import multiprocessing
from multiprocessing import Process, Pipe, Queue
from itertools import izip  


def spawn(f): 
	"""
	Helper function for `parmap` which prepares the worker `f` to be spawned.
	""" 
	def fun(pipe,x):  
		pipe.send(f(x))  
		pipe.close()  
	return fun  

def parmap(f, X, ncpu):  
	"""
	This is an alternative to multiprocessing.Pool to avoid the limitations of the package (pickling stuff...)
	
	.. note:: see http://stackoverflow.com/questions/3288595/multiprocessing-how-to-use-pool-map-on-a-function-defined-in-a-class
	
	.. note:: It is very possible that multiprocessing.Pool is fixed in python3 
	"""
	pipe=[Pipe() for x in X]  
	processes=[Process(target=spawn(f),args=(c,x)) for x,(p,c) in izip(X,pipe)]  
	numProcesses = len(processes)  
	processNum = 0  
	outputList = []  
	while processNum < numProcesses:  
		endProcessNum = min(processNum+ncpu, numProcesses)  
		for proc in processes[processNum:endProcessNum]:  
			proc.start()  
		# It is crucial to call recv() before join() to avoid deadlocks !
		for proc,c in pipe[processNum:endProcessNum]:  
			outputList.append(proc.recv())  
		for proc in processes[processNum:endProcessNum]:  
			proc.join()  

		processNum = endProcessNum  
	return outputList	


if __name__ == '__main__':
	print(parmap(lambda i: i * 2, [1, 2, 3, 4, 6, 7, 8], 3))





import matplotlib
matplotlib.use("AGG")

import momentsml.tools
import momentsml.learn
import momentsml

import config
import numpy as np
import os


import logging
logging.basicConfig(format=config.loggerformat, level=logging.DEBUG)
logger = logging.getLogger(__name__)



for subfield in config.great3.subfields:

	logger.info("Working on subfield {}".format(subfield))

	meascatpath = config.great3.subpath(subfield, "obs", "img_meascat.pkl")
	cat = momentsml.tools.io.readpickle(meascatpath)
	
	# Predicting shear
	traindir = config.great3.subpath(subfield, "ml", config.datasets["train-shear"])
	cat = momentsml.learn.tenbilacrun.predict(cat, config.shearconflist, traindir)

	# Predicting weight
	traindir = config.great3.subpath(subfield, "ml", config.datasets["train-weight"])
	cat = momentsml.learn.tenbilacrun.predict(cat, config.weightconflist, traindir)

	
	predcatpath = config.great3.subpath(subfield, "pred", "predcat_{}.pkl".format(config.predcode))
	momentsml.tools.io.writepickle(cat, predcatpath)
	

	"""
	# If predictions are masked, put them to 20 and weights to 0:	
	cat["pre_s1"][cat["pre_s1"].mask] = 20.0
	cat["pre_s2"][cat["pre_s2"].mask] = 20.0
	cat["pre_s1w"][cat["pre_s1"].mask] = 0.0
	cat["pre_s2w"][cat["pre_s2"].mask] = 0.0

	cat["pre_s1"][cat["pre_s1w"].mask] = 20.0
	cat["pre_s2"][cat["pre_s2w"].mask] = 20.0
	cat["pre_s1w"][cat["pre_s1w"].mask] = 0.0
	cat["pre_s2w"][cat["pre_s2w"].mask] = 0.0
	
	
	# We cut out the columns we need
	preobscat = cat["ID","pre_s1","pre_s1w", "pre_s2","pre_s2w"]
	
	# We write the ascii file
	preobscat.write(great3.subpath(subfield, "out", "%03i.cat" % subfield), format="ascii.commented_header")
	
	logger.info("Wrote predictions cat for subfield %03i" % subfield)
	"""
